# coding: utf-8

"""
    Camunda BPM REST API

    OpenApi Spec for Camunda BPM REST API.  # noqa: E501

    The version of the OpenAPI document: 7.14.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from generic_camunda_client.configuration import Configuration


class CommentDto(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'user_id': 'str',
        'task_id': 'str',
        'time': 'datetime',
        'message': 'str',
        'removal_time': 'datetime',
        'root_process_instance_id': 'str',
        'links': 'list[AtomLink]'
    }

    attribute_map = {
        'id': 'id',
        'user_id': 'userId',
        'task_id': 'taskId',
        'time': 'time',
        'message': 'message',
        'removal_time': 'removalTime',
        'root_process_instance_id': 'rootProcessInstanceId',
        'links': 'links'
    }

    def __init__(self, id=None, user_id=None, task_id=None, time=None, message=None, removal_time=None, root_process_instance_id=None, links=None, local_vars_configuration=None):  # noqa: E501
        """CommentDto - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._user_id = None
        self._task_id = None
        self._time = None
        self._message = None
        self._removal_time = None
        self._root_process_instance_id = None
        self._links = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if user_id is not None:
            self.user_id = user_id
        if task_id is not None:
            self.task_id = task_id
        if time is not None:
            self.time = time
        if message is not None:
            self.message = message
        self.removal_time = removal_time
        if root_process_instance_id is not None:
            self.root_process_instance_id = root_process_instance_id
        if links is not None:
            self.links = links

    @property
    def id(self):
        """Gets the id of this CommentDto.  # noqa: E501

        The id of the task comment.  # noqa: E501

        :return: The id of this CommentDto.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this CommentDto.

        The id of the task comment.  # noqa: E501

        :param id: The id of this CommentDto.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def user_id(self):
        """Gets the user_id of this CommentDto.  # noqa: E501

        The id of the user who created the comment.  # noqa: E501

        :return: The user_id of this CommentDto.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this CommentDto.

        The id of the user who created the comment.  # noqa: E501

        :param user_id: The user_id of this CommentDto.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    @property
    def task_id(self):
        """Gets the task_id of this CommentDto.  # noqa: E501

        The id of the task to which the comment belongs.  # noqa: E501

        :return: The task_id of this CommentDto.  # noqa: E501
        :rtype: str
        """
        return self._task_id

    @task_id.setter
    def task_id(self, task_id):
        """Sets the task_id of this CommentDto.

        The id of the task to which the comment belongs.  # noqa: E501

        :param task_id: The task_id of this CommentDto.  # noqa: E501
        :type: str
        """

        self._task_id = task_id

    @property
    def time(self):
        """Gets the time of this CommentDto.  # noqa: E501

        The time when the comment was created. [Default format]($(docsUrl)/reference/rest/overview/date-format/) `yyyy-MM-dd'T'HH:mm:ss.SSSZ`.  # noqa: E501

        :return: The time of this CommentDto.  # noqa: E501
        :rtype: datetime
        """
        return self._time

    @time.setter
    def time(self, time):
        """Sets the time of this CommentDto.

        The time when the comment was created. [Default format]($(docsUrl)/reference/rest/overview/date-format/) `yyyy-MM-dd'T'HH:mm:ss.SSSZ`.  # noqa: E501

        :param time: The time of this CommentDto.  # noqa: E501
        :type: datetime
        """

        self._time = time

    @property
    def message(self):
        """Gets the message of this CommentDto.  # noqa: E501

        The content of the comment.  # noqa: E501

        :return: The message of this CommentDto.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this CommentDto.

        The content of the comment.  # noqa: E501

        :param message: The message of this CommentDto.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def removal_time(self):
        """Gets the removal_time of this CommentDto.  # noqa: E501

        The time after which the comment should be removed by the History Cleanup job. [Default format]($(docsUrl)/reference/rest/overview/date-format/) `yyyy-MM-dd'T'HH:mm:ss.SSSZ`.  # noqa: E501

        :return: The removal_time of this CommentDto.  # noqa: E501
        :rtype: datetime
        """
        return self._removal_time

    @removal_time.setter
    def removal_time(self, removal_time):
        """Sets the removal_time of this CommentDto.

        The time after which the comment should be removed by the History Cleanup job. [Default format]($(docsUrl)/reference/rest/overview/date-format/) `yyyy-MM-dd'T'HH:mm:ss.SSSZ`.  # noqa: E501

        :param removal_time: The removal_time of this CommentDto.  # noqa: E501
        :type: datetime
        """

        self._removal_time = removal_time

    @property
    def root_process_instance_id(self):
        """Gets the root_process_instance_id of this CommentDto.  # noqa: E501

        The process instance id of the root process instance that initiated the process containing the task.  # noqa: E501

        :return: The root_process_instance_id of this CommentDto.  # noqa: E501
        :rtype: str
        """
        return self._root_process_instance_id

    @root_process_instance_id.setter
    def root_process_instance_id(self, root_process_instance_id):
        """Sets the root_process_instance_id of this CommentDto.

        The process instance id of the root process instance that initiated the process containing the task.  # noqa: E501

        :param root_process_instance_id: The root_process_instance_id of this CommentDto.  # noqa: E501
        :type: str
        """

        self._root_process_instance_id = root_process_instance_id

    @property
    def links(self):
        """Gets the links of this CommentDto.  # noqa: E501

        The links associated to this resource, with `method`, `href` and `rel`.  # noqa: E501

        :return: The links of this CommentDto.  # noqa: E501
        :rtype: list[AtomLink]
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this CommentDto.

        The links associated to this resource, with `method`, `href` and `rel`.  # noqa: E501

        :param links: The links of this CommentDto.  # noqa: E501
        :type: list[AtomLink]
        """

        self._links = links

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CommentDto):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CommentDto):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Camunda BPM REST API

    OpenApi Spec for Camunda BPM REST API.  # noqa: E501

    The version of the OpenAPI document: 7.14.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from generic_camunda_client.configuration import Configuration


class HistoricActivityInstanceDto(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'parent_activity_instance_id': 'str',
        'activity_id': 'str',
        'activity_name': 'str',
        'activity_type': 'str',
        'process_definition_key': 'str',
        'process_definition_id': 'str',
        'process_instance_id': 'str',
        'execution_id': 'str',
        'task_id': 'str',
        'assignee': 'str',
        'called_process_instance_id': 'str',
        'called_case_instance_id': 'str',
        'start_time': 'datetime',
        'end_time': 'datetime',
        'duration_in_millis': 'int',
        'canceled': 'bool',
        'complete_scope': 'bool',
        'tenant_id': 'str',
        'removal_time': 'datetime',
        'root_process_instance_id': 'str'
    }

    attribute_map = {
        'id': 'id',
        'parent_activity_instance_id': 'parentActivityInstanceId',
        'activity_id': 'activityId',
        'activity_name': 'activityName',
        'activity_type': 'activityType',
        'process_definition_key': 'processDefinitionKey',
        'process_definition_id': 'processDefinitionId',
        'process_instance_id': 'processInstanceId',
        'execution_id': 'executionId',
        'task_id': 'taskId',
        'assignee': 'assignee',
        'called_process_instance_id': 'calledProcessInstanceId',
        'called_case_instance_id': 'calledCaseInstanceId',
        'start_time': 'startTime',
        'end_time': 'endTime',
        'duration_in_millis': 'durationInMillis',
        'canceled': 'canceled',
        'complete_scope': 'completeScope',
        'tenant_id': 'tenantId',
        'removal_time': 'removalTime',
        'root_process_instance_id': 'rootProcessInstanceId'
    }

    def __init__(self, id=None, parent_activity_instance_id=None, activity_id=None, activity_name=None, activity_type=None, process_definition_key=None, process_definition_id=None, process_instance_id=None, execution_id=None, task_id=None, assignee=None, called_process_instance_id=None, called_case_instance_id=None, start_time=None, end_time=None, duration_in_millis=None, canceled=None, complete_scope=None, tenant_id=None, removal_time=None, root_process_instance_id=None, local_vars_configuration=None):  # noqa: E501
        """HistoricActivityInstanceDto - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._parent_activity_instance_id = None
        self._activity_id = None
        self._activity_name = None
        self._activity_type = None
        self._process_definition_key = None
        self._process_definition_id = None
        self._process_instance_id = None
        self._execution_id = None
        self._task_id = None
        self._assignee = None
        self._called_process_instance_id = None
        self._called_case_instance_id = None
        self._start_time = None
        self._end_time = None
        self._duration_in_millis = None
        self._canceled = None
        self._complete_scope = None
        self._tenant_id = None
        self._removal_time = None
        self._root_process_instance_id = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if parent_activity_instance_id is not None:
            self.parent_activity_instance_id = parent_activity_instance_id
        if activity_id is not None:
            self.activity_id = activity_id
        if activity_name is not None:
            self.activity_name = activity_name
        if activity_type is not None:
            self.activity_type = activity_type
        if process_definition_key is not None:
            self.process_definition_key = process_definition_key
        if process_definition_id is not None:
            self.process_definition_id = process_definition_id
        if process_instance_id is not None:
            self.process_instance_id = process_instance_id
        if execution_id is not None:
            self.execution_id = execution_id
        if task_id is not None:
            self.task_id = task_id
        if assignee is not None:
            self.assignee = assignee
        if called_process_instance_id is not None:
            self.called_process_instance_id = called_process_instance_id
        if called_case_instance_id is not None:
            self.called_case_instance_id = called_case_instance_id
        self.start_time = start_time
        self.end_time = end_time
        if duration_in_millis is not None:
            self.duration_in_millis = duration_in_millis
        self.canceled = canceled
        self.complete_scope = complete_scope
        if tenant_id is not None:
            self.tenant_id = tenant_id
        self.removal_time = removal_time
        if root_process_instance_id is not None:
            self.root_process_instance_id = root_process_instance_id

    @property
    def id(self):
        """Gets the id of this HistoricActivityInstanceDto.  # noqa: E501

        The id of the activity instance.  # noqa: E501

        :return: The id of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this HistoricActivityInstanceDto.

        The id of the activity instance.  # noqa: E501

        :param id: The id of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def parent_activity_instance_id(self):
        """Gets the parent_activity_instance_id of this HistoricActivityInstanceDto.  # noqa: E501

        The id of the parent activity instance, for example a sub process instance.  # noqa: E501

        :return: The parent_activity_instance_id of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._parent_activity_instance_id

    @parent_activity_instance_id.setter
    def parent_activity_instance_id(self, parent_activity_instance_id):
        """Sets the parent_activity_instance_id of this HistoricActivityInstanceDto.

        The id of the parent activity instance, for example a sub process instance.  # noqa: E501

        :param parent_activity_instance_id: The parent_activity_instance_id of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._parent_activity_instance_id = parent_activity_instance_id

    @property
    def activity_id(self):
        """Gets the activity_id of this HistoricActivityInstanceDto.  # noqa: E501

        The id of the activity that this object is an instance of.  # noqa: E501

        :return: The activity_id of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._activity_id

    @activity_id.setter
    def activity_id(self, activity_id):
        """Sets the activity_id of this HistoricActivityInstanceDto.

        The id of the activity that this object is an instance of.  # noqa: E501

        :param activity_id: The activity_id of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._activity_id = activity_id

    @property
    def activity_name(self):
        """Gets the activity_name of this HistoricActivityInstanceDto.  # noqa: E501

        The name of the activity that this object is an instance of.  # noqa: E501

        :return: The activity_name of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._activity_name

    @activity_name.setter
    def activity_name(self, activity_name):
        """Sets the activity_name of this HistoricActivityInstanceDto.

        The name of the activity that this object is an instance of.  # noqa: E501

        :param activity_name: The activity_name of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._activity_name = activity_name

    @property
    def activity_type(self):
        """Gets the activity_type of this HistoricActivityInstanceDto.  # noqa: E501

        The type of the activity that this object is an instance of.  # noqa: E501

        :return: The activity_type of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._activity_type

    @activity_type.setter
    def activity_type(self, activity_type):
        """Sets the activity_type of this HistoricActivityInstanceDto.

        The type of the activity that this object is an instance of.  # noqa: E501

        :param activity_type: The activity_type of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._activity_type = activity_type

    @property
    def process_definition_key(self):
        """Gets the process_definition_key of this HistoricActivityInstanceDto.  # noqa: E501

        The key of the process definition that this activity instance belongs to.  # noqa: E501

        :return: The process_definition_key of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._process_definition_key

    @process_definition_key.setter
    def process_definition_key(self, process_definition_key):
        """Sets the process_definition_key of this HistoricActivityInstanceDto.

        The key of the process definition that this activity instance belongs to.  # noqa: E501

        :param process_definition_key: The process_definition_key of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._process_definition_key = process_definition_key

    @property
    def process_definition_id(self):
        """Gets the process_definition_id of this HistoricActivityInstanceDto.  # noqa: E501

        The id of the process definition that this activity instance belongs to.  # noqa: E501

        :return: The process_definition_id of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._process_definition_id

    @process_definition_id.setter
    def process_definition_id(self, process_definition_id):
        """Sets the process_definition_id of this HistoricActivityInstanceDto.

        The id of the process definition that this activity instance belongs to.  # noqa: E501

        :param process_definition_id: The process_definition_id of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._process_definition_id = process_definition_id

    @property
    def process_instance_id(self):
        """Gets the process_instance_id of this HistoricActivityInstanceDto.  # noqa: E501

        The id of the process instance that this activity instance belongs to.  # noqa: E501

        :return: The process_instance_id of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._process_instance_id

    @process_instance_id.setter
    def process_instance_id(self, process_instance_id):
        """Sets the process_instance_id of this HistoricActivityInstanceDto.

        The id of the process instance that this activity instance belongs to.  # noqa: E501

        :param process_instance_id: The process_instance_id of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._process_instance_id = process_instance_id

    @property
    def execution_id(self):
        """Gets the execution_id of this HistoricActivityInstanceDto.  # noqa: E501

        The id of the execution that executed this activity instance.  # noqa: E501

        :return: The execution_id of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._execution_id

    @execution_id.setter
    def execution_id(self, execution_id):
        """Sets the execution_id of this HistoricActivityInstanceDto.

        The id of the execution that executed this activity instance.  # noqa: E501

        :param execution_id: The execution_id of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._execution_id = execution_id

    @property
    def task_id(self):
        """Gets the task_id of this HistoricActivityInstanceDto.  # noqa: E501

        The id of the task that is associated to this activity instance. Is only set if the activity is a user task.  # noqa: E501

        :return: The task_id of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._task_id

    @task_id.setter
    def task_id(self, task_id):
        """Sets the task_id of this HistoricActivityInstanceDto.

        The id of the task that is associated to this activity instance. Is only set if the activity is a user task.  # noqa: E501

        :param task_id: The task_id of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._task_id = task_id

    @property
    def assignee(self):
        """Gets the assignee of this HistoricActivityInstanceDto.  # noqa: E501

        The assignee of the task that is associated to this activity instance. Is only set if the activity is a user task.  # noqa: E501

        :return: The assignee of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._assignee

    @assignee.setter
    def assignee(self, assignee):
        """Sets the assignee of this HistoricActivityInstanceDto.

        The assignee of the task that is associated to this activity instance. Is only set if the activity is a user task.  # noqa: E501

        :param assignee: The assignee of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._assignee = assignee

    @property
    def called_process_instance_id(self):
        """Gets the called_process_instance_id of this HistoricActivityInstanceDto.  # noqa: E501

        The id of the called process instance. Is only set if the activity is a call activity and the called instance a process instance.  # noqa: E501

        :return: The called_process_instance_id of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._called_process_instance_id

    @called_process_instance_id.setter
    def called_process_instance_id(self, called_process_instance_id):
        """Sets the called_process_instance_id of this HistoricActivityInstanceDto.

        The id of the called process instance. Is only set if the activity is a call activity and the called instance a process instance.  # noqa: E501

        :param called_process_instance_id: The called_process_instance_id of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._called_process_instance_id = called_process_instance_id

    @property
    def called_case_instance_id(self):
        """Gets the called_case_instance_id of this HistoricActivityInstanceDto.  # noqa: E501

        The id of the called case instance. Is only set if the activity is a call activity and the called instance a case instance.  # noqa: E501

        :return: The called_case_instance_id of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._called_case_instance_id

    @called_case_instance_id.setter
    def called_case_instance_id(self, called_case_instance_id):
        """Sets the called_case_instance_id of this HistoricActivityInstanceDto.

        The id of the called case instance. Is only set if the activity is a call activity and the called instance a case instance.  # noqa: E501

        :param called_case_instance_id: The called_case_instance_id of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._called_case_instance_id = called_case_instance_id

    @property
    def start_time(self):
        """Gets the start_time of this HistoricActivityInstanceDto.  # noqa: E501

        The time the instance was started. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.  # noqa: E501

        :return: The start_time of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: datetime
        """
        return self._start_time

    @start_time.setter
    def start_time(self, start_time):
        """Sets the start_time of this HistoricActivityInstanceDto.

        The time the instance was started. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.  # noqa: E501

        :param start_time: The start_time of this HistoricActivityInstanceDto.  # noqa: E501
        :type: datetime
        """

        self._start_time = start_time

    @property
    def end_time(self):
        """Gets the end_time of this HistoricActivityInstanceDto.  # noqa: E501

        The time the instance ended. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.  # noqa: E501

        :return: The end_time of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: datetime
        """
        return self._end_time

    @end_time.setter
    def end_time(self, end_time):
        """Sets the end_time of this HistoricActivityInstanceDto.

        The time the instance ended. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.  # noqa: E501

        :param end_time: The end_time of this HistoricActivityInstanceDto.  # noqa: E501
        :type: datetime
        """

        self._end_time = end_time

    @property
    def duration_in_millis(self):
        """Gets the duration_in_millis of this HistoricActivityInstanceDto.  # noqa: E501

        The time the instance took to finish (in milliseconds).  # noqa: E501

        :return: The duration_in_millis of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: int
        """
        return self._duration_in_millis

    @duration_in_millis.setter
    def duration_in_millis(self, duration_in_millis):
        """Sets the duration_in_millis of this HistoricActivityInstanceDto.

        The time the instance took to finish (in milliseconds).  # noqa: E501

        :param duration_in_millis: The duration_in_millis of this HistoricActivityInstanceDto.  # noqa: E501
        :type: int
        """

        self._duration_in_millis = duration_in_millis

    @property
    def canceled(self):
        """Gets the canceled of this HistoricActivityInstanceDto.  # noqa: E501

        If `true`, this activity instance is canceled.  # noqa: E501

        :return: The canceled of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: bool
        """
        return self._canceled

    @canceled.setter
    def canceled(self, canceled):
        """Sets the canceled of this HistoricActivityInstanceDto.

        If `true`, this activity instance is canceled.  # noqa: E501

        :param canceled: The canceled of this HistoricActivityInstanceDto.  # noqa: E501
        :type: bool
        """

        self._canceled = canceled

    @property
    def complete_scope(self):
        """Gets the complete_scope of this HistoricActivityInstanceDto.  # noqa: E501

        If `true`, this activity instance did complete a BPMN 2.0 scope.  # noqa: E501

        :return: The complete_scope of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: bool
        """
        return self._complete_scope

    @complete_scope.setter
    def complete_scope(self, complete_scope):
        """Sets the complete_scope of this HistoricActivityInstanceDto.

        If `true`, this activity instance did complete a BPMN 2.0 scope.  # noqa: E501

        :param complete_scope: The complete_scope of this HistoricActivityInstanceDto.  # noqa: E501
        :type: bool
        """

        self._complete_scope = complete_scope

    @property
    def tenant_id(self):
        """Gets the tenant_id of this HistoricActivityInstanceDto.  # noqa: E501

        The tenant id of the activity instance.  # noqa: E501

        :return: The tenant_id of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._tenant_id

    @tenant_id.setter
    def tenant_id(self, tenant_id):
        """Sets the tenant_id of this HistoricActivityInstanceDto.

        The tenant id of the activity instance.  # noqa: E501

        :param tenant_id: The tenant_id of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._tenant_id = tenant_id

    @property
    def removal_time(self):
        """Gets the removal_time of this HistoricActivityInstanceDto.  # noqa: E501

        The time after which the activity instance should be removed by the History Cleanup job. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.  # noqa: E501

        :return: The removal_time of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: datetime
        """
        return self._removal_time

    @removal_time.setter
    def removal_time(self, removal_time):
        """Sets the removal_time of this HistoricActivityInstanceDto.

        The time after which the activity instance should be removed by the History Cleanup job. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.  # noqa: E501

        :param removal_time: The removal_time of this HistoricActivityInstanceDto.  # noqa: E501
        :type: datetime
        """

        self._removal_time = removal_time

    @property
    def root_process_instance_id(self):
        """Gets the root_process_instance_id of this HistoricActivityInstanceDto.  # noqa: E501

        The process instance id of the root process instance that initiated the process containing this activity instance.  # noqa: E501

        :return: The root_process_instance_id of this HistoricActivityInstanceDto.  # noqa: E501
        :rtype: str
        """
        return self._root_process_instance_id

    @root_process_instance_id.setter
    def root_process_instance_id(self, root_process_instance_id):
        """Sets the root_process_instance_id of this HistoricActivityInstanceDto.

        The process instance id of the root process instance that initiated the process containing this activity instance.  # noqa: E501

        :param root_process_instance_id: The root_process_instance_id of this HistoricActivityInstanceDto.  # noqa: E501
        :type: str
        """

        self._root_process_instance_id = root_process_instance_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, HistoricActivityInstanceDto):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, HistoricActivityInstanceDto):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Camunda BPM REST API

    OpenApi Spec for Camunda BPM REST API.  # noqa: E501

    The version of the OpenAPI document: 7.14.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest
import datetime

import generic_camunda_client
from generic_camunda_client.models.historic_activity_instance_dto import HistoricActivityInstanceDto  # noqa: E501
from generic_camunda_client.rest import ApiException

class TestHistoricActivityInstanceDto(unittest.TestCase):
    """HistoricActivityInstanceDto unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional):
        """Test HistoricActivityInstanceDto
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # model = generic_camunda_client.models.historic_activity_instance_dto.HistoricActivityInstanceDto()  # noqa: E501
        if include_optional :
            return HistoricActivityInstanceDto(
                id = '0', 
                parent_activity_instance_id = '0', 
                activity_id = '0', 
                activity_name = '0', 
                activity_type = '0', 
                process_definition_key = '0', 
                process_definition_id = '0', 
                process_instance_id = '0', 
                execution_id = '0', 
                task_id = '0', 
                assignee = '0', 
                called_process_instance_id = '0', 
                called_case_instance_id = '0', 
                start_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                end_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                duration_in_millis = 56, 
                canceled = True, 
                complete_scope = True, 
                tenant_id = '0', 
                removal_time = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                root_process_instance_id = '0'
            )
        else :
            return HistoricActivityInstanceDto(
        )

    def testHistoricActivityInstanceDto(self):
        """Test HistoricActivityInstanceDto"""
        inst_req_only = self.make_instance(include_optional=False)
        inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Camunda BPM REST API

    OpenApi Spec for Camunda BPM REST API.  # noqa: E501

    The version of the OpenAPI document: 7.14.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest
import datetime

import generic_camunda_client
from generic_camunda_client.models.task_query_dto import TaskQueryDto  # noqa: E501
from generic_camunda_client.rest import ApiException

class TestTaskQueryDto(unittest.TestCase):
    """TaskQueryDto unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional):
        """Test TaskQueryDto
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # model = generic_camunda_client.models.task_query_dto.TaskQueryDto()  # noqa: E501
        if include_optional :
            return TaskQueryDto(
                process_instance_id = '0', 
                process_instance_id_in = [
                    '0'
                    ], 
                process_instance_business_key = '0', 
                process_instance_business_key_expression = '0', 
                process_instance_business_key_in = [
                    '0'
                    ], 
                process_instance_business_key_like = '0', 
                process_instance_business_key_like_expression = '0', 
                process_definition_id = '0', 
                process_definition_key = '0', 
                process_definition_key_in = [
                    '0'
                    ], 
                process_definition_name = '0', 
                process_definition_name_like = '0', 
                execution_id = '0', 
                case_instance_id = '0', 
                case_instance_business_key = '0', 
                case_instance_business_key_like = '0', 
                case_definition_id = '0', 
                case_definition_key = '0', 
                case_definition_name = '0', 
                case_definition_name_like = '0', 
                case_execution_id = '0', 
                activity_instance_id_in = [
                    '0'
                    ], 
                tenant_id_in = [
                    '0'
                    ], 
                without_tenant_id = True, 
                assignee = '0', 
                assignee_expression = '0', 
                assignee_like = '0', 
                assignee_like_expression = '0', 
                assignee_in = [
                    '0'
                    ], 
                owner = '0', 
                owner_expression = '0', 
                candidate_group = '0', 
                candidate_group_expression = '0', 
                candidate_user = '0', 
                candidate_user_expression = '0', 
                include_assigned_tasks = True, 
                involved_user = '0', 
                involved_user_expression = '0', 
                assigned = True, 
                unassigned = True, 
                task_definition_key = '0', 
                task_definition_key_in = [
                    '0'
                    ], 
                task_definition_key_like = '0', 
                name = '0', 
                name_not_equal = '0', 
                name_like = '0', 
                name_not_like = '0', 
                description = '0', 
                description_like = '0', 
                priority = 56, 
                max_priority = 56, 
                min_priority = 56, 
                due_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                due_date_expression = '0', 
                due_after = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                due_after_expression = '0', 
                due_before = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                due_before_expression = '0', 
                follow_up_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                follow_up_date_expression = '0', 
                follow_up_after = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                follow_up_after_expression = '0', 
                follow_up_before = '0', 
                follow_up_before_expression = '0', 
                follow_up_before_or_not_existent = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                follow_up_before_or_not_existent_expression = '0', 
                created_on = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                created_on_expression = '0', 
                created_after = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                created_after_expression = '0', 
                created_before = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                created_before_expression = '0', 
                delegation_state = 'PENDING', 
                candidate_groups = [
                    '0'
                    ], 
                candidate_groups_expression = '0', 
                with_candidate_groups = True, 
                without_candidate_groups = True, 
                with_candidate_users = True, 
                without_candidate_users = True, 
                active = True, 
                suspended = True, 
                task_variables = [
                    generic_camunda_client.models.variable_query_parameter_dto.VariableQueryParameterDto(
                        name = '0', 
                        operator = 'eq', 
                        value = generic_camunda_client.models.value.value(), )
                    ], 
                process_variables = [
                    generic_camunda_client.models.variable_query_parameter_dto.VariableQueryParameterDto(
                        name = '0', 
                        operator = 'eq', 
                        value = generic_camunda_client.models.value.value(), )
                    ], 
                case_instance_variables = [
                    generic_camunda_client.models.variable_query_parameter_dto.VariableQueryParameterDto(
                        name = '0', 
                        operator = 'eq', 
                        value = generic_camunda_client.models.value.value(), )
                    ], 
                variable_names_ignore_case = True, 
                variable_values_ignore_case = True, 
                parent_task_id = '0', 
                or_queries = [
                    generic_camunda_client.models.task_query_dto.TaskQueryDto(
                        process_instance_id = '0', 
                        process_instance_id_in = [
                            '0'
                            ], 
                        process_instance_business_key = '0', 
                        process_instance_business_key_expression = '0', 
                        process_instance_business_key_in = [
                            '0'
                            ], 
                        process_instance_business_key_like = '0', 
                        process_instance_business_key_like_expression = '0', 
                        process_definition_id = '0', 
                        process_definition_key = '0', 
                        process_definition_key_in = [
                            '0'
                            ], 
                        process_definition_name = '0', 
                        process_definition_name_like = '0', 
                        execution_id = '0', 
                        case_instance_id = '0', 
                        case_instance_business_key = '0', 
                        case_instance_business_key_like = '0', 
                        case_definition_id = '0', 
                        case_definition_key = '0', 
                        case_definition_name = '0', 
                        case_definition_name_like = '0', 
                        case_execution_id = '0', 
                        activity_instance_id_in = [
                            '0'
                            ], 
                        tenant_id_in = [
                            '0'
                            ], 
                        without_tenant_id = True, 
                        assignee = '0', 
                        assignee_expression = '0', 
                        assignee_like = '0', 
                        assignee_like_expression = '0', 
                        assignee_in = [
                            '0'
                            ], 
                        owner = '0', 
                        owner_expression = '0', 
                        candidate_group = '0', 
                        candidate_group_expression = '0', 
                        candidate_user = '0', 
                        candidate_user_expression = '0', 
                        include_assigned_tasks = True, 
                        involved_user = '0', 
                        involved_user_expression = '0', 
                        assigned = True, 
                        unassigned = True, 
                        task_definition_key = '0', 
                        task_definition_key_in = [
                            '0'
                            ], 
                        task_definition_key_like = '0', 
                        name = '0', 
                        name_not_equal = '0', 
                        name_like = '0', 
                        name_not_like = '0', 
                        description = '0', 
                        description_like = '0', 
                        priority = 56, 
                        max_priority = 56, 
                        min_priority = 56, 
                        due_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        due_date_expression = '0', 
                        due_after = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        due_after_expression = '0', 
                        due_before = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        due_before_expression = '0', 
                        follow_up_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        follow_up_date_expression = '0', 
                        follow_up_after = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        follow_up_after_expression = '0', 
                        follow_up_before = '0', 
                        follow_up_before_expression = '0', 
                        follow_up_before_or_not_existent = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        follow_up_before_or_not_existent_expression = '0', 
                        created_on = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        created_on_expression = '0', 
                        created_after = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        created_after_expression = '0', 
                        created_before = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        created_before_expression = '0', 
                        delegation_state = 'PENDING', 
                        candidate_groups = [
                            '0'
                            ], 
                        candidate_groups_expression = '0', 
                        with_candidate_groups = True, 
                        without_candidate_groups = True, 
                        with_candidate_users = True, 
                        without_candidate_users = True, 
                        active = True, 
                        suspended = True, 
                        task_variables = [
                            generic_camunda_client.models.variable_query_parameter_dto.VariableQueryParameterDto(
                                name = '0', 
                                operator = 'eq', 
                                value = generic_camunda_client.models.value.value(), )
                            ], 
                        process_variables = [
                            generic_camunda_client.models.variable_query_parameter_dto.VariableQueryParameterDto(
                                name = '0', 
                                operator = 'eq', 
                                value = generic_camunda_client.models.value.value(), )
                            ], 
                        case_instance_variables = [
                            generic_camunda_client.models.variable_query_parameter_dto.VariableQueryParameterDto(
                                name = '0', 
                                operator = 'eq', 
                                value = generic_camunda_client.models.value.value(), )
                            ], 
                        variable_names_ignore_case = True, 
                        variable_values_ignore_case = True, 
                        parent_task_id = '0', 
                        or_queries = [
                            generic_camunda_client.models.task_query_dto.TaskQueryDto(
                                process_instance_id = '0', 
                                process_instance_business_key = '0', 
                                process_instance_business_key_expression = '0', 
                                process_instance_business_key_like = '0', 
                                process_instance_business_key_like_expression = '0', 
                                process_definition_id = '0', 
                                process_definition_key = '0', 
                                process_definition_name = '0', 
                                process_definition_name_like = '0', 
                                execution_id = '0', 
                                case_instance_id = '0', 
                                case_instance_business_key = '0', 
                                case_instance_business_key_like = '0', 
                                case_definition_id = '0', 
                                case_definition_key = '0', 
                                case_definition_name = '0', 
                                case_definition_name_like = '0', 
                                case_execution_id = '0', 
                                without_tenant_id = True, 
                                assignee = '0', 
                                assignee_expression = '0', 
                                assignee_like = '0', 
                                assignee_like_expression = '0', 
                                owner = '0', 
                                owner_expression = '0', 
                                candidate_group = '0', 
                                candidate_group_expression = '0', 
                                candidate_user = '0', 
                                candidate_user_expression = '0', 
                                include_assigned_tasks = True, 
                                involved_user = '0', 
                                involved_user_expression = '0', 
                                assigned = True, 
                                unassigned = True, 
                                task_definition_key = '0', 
                                task_definition_key_like = '0', 
                                name = '0', 
                                name_not_equal = '0', 
                                name_like = '0', 
                                name_not_like = '0', 
                                description = '0', 
                                description_like = '0', 
                                priority = 56, 
                                max_priority = 56, 
                                min_priority = 56, 
                                due_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                due_date_expression = '0', 
                                due_after = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                due_after_expression = '0', 
                                due_before = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                due_before_expression = '0', 
                                follow_up_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                follow_up_date_expression = '0', 
                                follow_up_after = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                follow_up_after_expression = '0', 
                                follow_up_before = '0', 
                                follow_up_before_expression = '0', 
                                follow_up_before_or_not_existent = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                follow_up_before_or_not_existent_expression = '0', 
                                created_on = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                created_on_expression = '0', 
                                created_after = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                created_after_expression = '0', 
                                created_before = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                created_before_expression = '0', 
                                delegation_state = 'PENDING', 
                                candidate_groups_expression = '0', 
                                with_candidate_groups = True, 
                                without_candidate_groups = True, 
                                with_candidate_users = True, 
                                without_candidate_users = True, 
                                active = True, 
                                suspended = True, 
                                variable_names_ignore_case = True, 
                                variable_values_ignore_case = True, 
                                parent_task_id = '0', 
                                sorting = [
                                    generic_camunda_client.models.task_query_dto_sorting.TaskQueryDto_sorting(
                                        sort_by = 'instanceId', 
                                        sort_order = 'asc', 
                                        parameters = generic_camunda_client.models.sort_task_query_parameters_dto.SortTaskQueryParametersDto(
                                            variable = '0', 
                                            type = '0', ), )
                                    ], )
                            ], 
                        sorting = [
                            generic_camunda_client.models.task_query_dto_sorting.TaskQueryDto_sorting(
                                sort_by = 'instanceId', 
                                sort_order = 'asc', )
                            ], )
                    ], 
                sorting = [
                    generic_camunda_client.models.task_query_dto_sorting.TaskQueryDto_sorting(
                        sort_by = 'instanceId', 
                        sort_order = 'asc', 
                        parameters = generic_camunda_client.models.sort_task_query_parameters_dto.SortTaskQueryParametersDto(
                            variable = '0', 
                            type = '0', ), )
                    ]
            )
        else :
            return TaskQueryDto(
        )

    def testTaskQueryDto(self):
        """Test TaskQueryDto"""
        inst_req_only = self.make_instance(include_optional=False)
        inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Camunda BPM REST API

    OpenApi Spec for Camunda BPM REST API.  # noqa: E501

    The version of the OpenAPI document: 7.14.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from generic_camunda_client.api_client import ApiClient
from generic_camunda_client.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class HistoricProcessInstanceApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def delete_historic_process_instance(self, id, **kwargs):  # noqa: E501
        """Delete  # noqa: E501

        Deletes a process instance from the history by id.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_historic_process_instance(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str id: The id of the historic process instance to be deleted. (required)
        :param bool fail_if_not_exists: If set to `false`, the request will still be successful if the process id is not found.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.delete_historic_process_instance_with_http_info(id, **kwargs)  # noqa: E501

    def delete_historic_process_instance_with_http_info(self, id, **kwargs):  # noqa: E501
        """Delete  # noqa: E501

        Deletes a process instance from the history by id.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_historic_process_instance_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str id: The id of the historic process instance to be deleted. (required)
        :param bool fail_if_not_exists: If set to `false`, the request will still be successful if the process id is not found.
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'id',
            'fail_if_not_exists'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_historic_process_instance" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in local_var_params or  # noqa: E501
                                                        local_var_params['id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `id` when calling `delete_historic_process_instance`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in local_var_params:
            path_params['id'] = local_var_params['id']  # noqa: E501

        query_params = []
        if 'fail_if_not_exists' in local_var_params and local_var_params['fail_if_not_exists'] is not None:  # noqa: E501
            query_params.append(('failIfNotExists', local_var_params['fail_if_not_exists']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/history/process-instance/{id}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_historic_process_instances_async(self, **kwargs):  # noqa: E501
        """Delete Async (POST)  # noqa: E501

        Delete multiple historic process instances asynchronously (batch). At least `historicProcessInstanceIds` or `historicProcessInstanceQuery` has to be provided. If both are provided then all instances matching query criterion and instances from the list will be deleted.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_historic_process_instances_async(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param DeleteHistoricProcessInstancesDto delete_historic_process_instances_dto:
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: BatchDto
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.delete_historic_process_instances_async_with_http_info(**kwargs)  # noqa: E501

    def delete_historic_process_instances_async_with_http_info(self, **kwargs):  # noqa: E501
        """Delete Async (POST)  # noqa: E501

        Delete multiple historic process instances asynchronously (batch). At least `historicProcessInstanceIds` or `historicProcessInstanceQuery` has to be provided. If both are provided then all instances matching query criterion and instances from the list will be deleted.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_historic_process_instances_async_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param DeleteHistoricProcessInstancesDto delete_historic_process_instances_dto:
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(BatchDto, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'delete_historic_process_instances_dto'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_historic_process_instances_async" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'delete_historic_process_instances_dto' in local_var_params:
            body_params = local_var_params['delete_historic_process_instances_dto']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/history/process-instance/delete', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='BatchDto',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_historic_variable_instances_of_historic_process_instance(self, id, **kwargs):  # noqa: E501
        """Delete Variable Instances  # noqa: E501

        Deletes all variables of a process instance from the history by id.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_historic_variable_instances_of_historic_process_instance(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str id: The id of the process instance for which all historic variables are to be deleted. (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.delete_historic_variable_instances_of_historic_process_instance_with_http_info(id, **kwargs)  # noqa: E501

    def delete_historic_variable_instances_of_historic_process_instance_with_http_info(self, id, **kwargs):  # noqa: E501
        """Delete Variable Instances  # noqa: E501

        Deletes all variables of a process instance from the history by id.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_historic_variable_instances_of_historic_process_instance_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str id: The id of the process instance for which all historic variables are to be deleted. (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'id'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_historic_variable_instances_of_historic_process_instance" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in local_var_params or  # noqa: E501
                                                        local_var_params['id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `id` when calling `delete_historic_variable_instances_of_historic_process_instance`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in local_var_params:
            path_params['id'] = local_var_params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/history/process-instance/{id}/variable-instances', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_historic_process_instance(self, id, **kwargs):  # noqa: E501
        """Get  # noqa: E501

        Retrieves a historic process instance by id, according to the `HistoricProcessInstance` interface in the engine.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_historic_process_instance(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str id: The id of the historic process instance to be retrieved. (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: HistoricProcessInstanceDto
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.get_historic_process_instance_with_http_info(id, **kwargs)  # noqa: E501

    def get_historic_process_instance_with_http_info(self, id, **kwargs):  # noqa: E501
        """Get  # noqa: E501

        Retrieves a historic process instance by id, according to the `HistoricProcessInstance` interface in the engine.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_historic_process_instance_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str id: The id of the historic process instance to be retrieved. (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(HistoricProcessInstanceDto, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'id'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_historic_process_instance" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in local_var_params or  # noqa: E501
                                                        local_var_params['id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `id` when calling `get_historic_process_instance`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in local_var_params:
            path_params['id'] = local_var_params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/history/process-instance/{id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='HistoricProcessInstanceDto',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_historic_process_instance_duration_report(self, report_type, period_unit, **kwargs):  # noqa: E501
        """Get Duration Report  # noqa: E501

        Retrieves a report about the duration of completed process instances, grouped by a period. These reports include the maximum, minimum and average duration of all completed process instances which were started in a given period.  **Note:** This only includes historic data.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_historic_process_instance_duration_report(report_type, period_unit, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str report_type: **Mandatory.** Specifies the type of the report to retrieve. To retrieve a report about the duration of process instances, the value must be set to `duration`. (required)
        :param str period_unit: **Mandatory.** Specifies the granularity of the report. Valid values are `month` and `quarter`. (required)
        :param str process_definition_id_in: Filter by process definition ids. Must be a comma-separated list of process definition ids.
        :param str process_definition_key_in: Filter by process definition keys. Must be a comma-separated list of process definition keys.
        :param datetime started_before: Restrict to instances that were started before the given date. By [default](), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2016-01-23T14:42:45.000+0200`.
        :param datetime started_after: Restrict to instances that were started after the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2016-01-23T14:42:45.000+0200`.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.get_historic_process_instance_duration_report_with_http_info(report_type, period_unit, **kwargs)  # noqa: E501

    def get_historic_process_instance_duration_report_with_http_info(self, report_type, period_unit, **kwargs):  # noqa: E501
        """Get Duration Report  # noqa: E501

        Retrieves a report about the duration of completed process instances, grouped by a period. These reports include the maximum, minimum and average duration of all completed process instances which were started in a given period.  **Note:** This only includes historic data.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_historic_process_instance_duration_report_with_http_info(report_type, period_unit, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str report_type: **Mandatory.** Specifies the type of the report to retrieve. To retrieve a report about the duration of process instances, the value must be set to `duration`. (required)
        :param str period_unit: **Mandatory.** Specifies the granularity of the report. Valid values are `month` and `quarter`. (required)
        :param str process_definition_id_in: Filter by process definition ids. Must be a comma-separated list of process definition ids.
        :param str process_definition_key_in: Filter by process definition keys. Must be a comma-separated list of process definition keys.
        :param datetime started_before: Restrict to instances that were started before the given date. By [default](), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2016-01-23T14:42:45.000+0200`.
        :param datetime started_after: Restrict to instances that were started after the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2016-01-23T14:42:45.000+0200`.
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(object, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'report_type',
            'period_unit',
            'process_definition_id_in',
            'process_definition_key_in',
            'started_before',
            'started_after'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_historic_process_instance_duration_report" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'report_type' is set
        if self.api_client.client_side_validation and ('report_type' not in local_var_params or  # noqa: E501
                                                        local_var_params['report_type'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `report_type` when calling `get_historic_process_instance_duration_report`")  # noqa: E501
        # verify the required parameter 'period_unit' is set
        if self.api_client.client_side_validation and ('period_unit' not in local_var_params or  # noqa: E501
                                                        local_var_params['period_unit'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `period_unit` when calling `get_historic_process_instance_duration_report`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'report_type' in local_var_params and local_var_params['report_type'] is not None:  # noqa: E501
            query_params.append(('reportType', local_var_params['report_type']))  # noqa: E501
        if 'period_unit' in local_var_params and local_var_params['period_unit'] is not None:  # noqa: E501
            query_params.append(('periodUnit', local_var_params['period_unit']))  # noqa: E501
        if 'process_definition_id_in' in local_var_params and local_var_params['process_definition_id_in'] is not None:  # noqa: E501
            query_params.append(('processDefinitionIdIn', local_var_params['process_definition_id_in']))  # noqa: E501
        if 'process_definition_key_in' in local_var_params and local_var_params['process_definition_key_in'] is not None:  # noqa: E501
            query_params.append(('processDefinitionKeyIn', local_var_params['process_definition_key_in']))  # noqa: E501
        if 'started_before' in local_var_params and local_var_params['started_before'] is not None:  # noqa: E501
            query_params.append(('startedBefore', local_var_params['started_before']))  # noqa: E501
        if 'started_after' in local_var_params and local_var_params['started_after'] is not None:  # noqa: E501
            query_params.append(('startedAfter', local_var_params['started_after']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['text/csv', 'application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/history/process-instance/report', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='object',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_historic_process_instances(self, **kwargs):  # noqa: E501
        """Get List  # noqa: E501

        Queries for historic process instances that fulfill the given parameters. The size of the result set can be retrieved by using the [Get Process Instance Count](https://docs.camunda.org/manual/7.14/reference/rest/history/process-instance/get-process-instance-query-count/) method.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_historic_process_instances(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str sort_by: Sort the results lexicographically by a given criterion. Must be used in conjunction with the sortOrder parameter.
        :param str sort_order: Sort the results in a given order. Values may be asc for ascending order or desc for descending order. Must be used in conjunction with the sortBy parameter.
        :param int first_result: Pagination of results. Specifies the index of the first result to return.
        :param int max_results: Pagination of results. Specifies the maximum number of results to return. Will return less results if there are no more results left.
        :param str process_instance_id: Filter by process instance id.
        :param str process_instance_ids: Filter by process instance ids. Filter by a comma-separated list of `Strings`.
        :param str process_definition_id: Filter by the process definition the instances run on.
        :param str process_definition_key: Filter by the key of the process definition the instances run on.
        :param str process_definition_key_in: Filter by a list of process definition keys. A process instance must have one of the given process definition keys. Filter by a comma-separated list of `Strings`.
        :param str process_definition_name: Filter by the name of the process definition the instances run on.
        :param str process_definition_name_like: Filter by process definition names that the parameter is a substring of.
        :param str process_definition_key_not_in: Exclude instances that belong to a set of process definitions. Filter by a comma-separated list of `Strings`.
        :param str process_instance_business_key: Filter by process instance business key.
        :param str process_instance_business_key_like: Filter by process instance business key that the parameter is a substring of.
        :param bool root_process_instances: Restrict the query to all process instances that are top level process instances.
        :param bool finished: Only include finished process instances. Value may only be `true`, as `false` is the default behavior.
        :param bool unfinished: Only include unfinished process instances. Value may only be `true`, as `false` is the default behavior.
        :param bool with_incidents: Only include process instances which have an incident. Value may only be `true`, as `false` is the default behavior.
        :param bool with_root_incidents: Only include process instances which have a root incident. Value may only be `true`, as `false` is the default behavior.
        :param str incident_type: Filter by the incident type. See the [User Guide](https://docs.camunda.org/manual/7.14/user-guide/process-engine/incidents/#incident-types) for a list of incident types.
        :param str incident_status: Only include process instances which have an incident in status either open or resolved. To get all process instances, use the query parameter withIncidents.
        :param str incident_message: Filter by the incident message. Exact match.
        :param str incident_message_like: Filter by the incident message that the parameter is a substring of.
        :param datetime started_before: Restrict to instances that were started before the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime started_after: Restrict to instances that were started after the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime finished_before: Restrict to instances that were finished before the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime finished_after: Restrict to instances that were finished after the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_activity_after: Restrict to instances that executed an activity after the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_activity_before: Restrict to instances that executed an activity before the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_job_after: Restrict to instances that executed an job after the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_job_before: Restrict to instances that executed an job before the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param str started_by: Only include process instances that were started by the given user.
        :param str super_process_instance_id: Restrict query to all process instances that are sub process instances of the given process instance. Takes a process instance id.
        :param str sub_process_instance_id: Restrict query to one process instance that has a sub process instance with the given id.
        :param str super_case_instance_id: Restrict query to all process instances that are sub process instances of the given case instance. Takes a case instance id.
        :param str sub_case_instance_id: Restrict query to one process instance that has a sub case instance with the given id.
        :param str case_instance_id: Restrict query to all process instances that are sub process instances of the given case instance. Takes a case instance id.
        :param str tenant_id_in: Filter by a list of tenant ids. A process instance must have one of the given tenant ids. Filter by a comma-separated list of `Strings`
        :param bool without_tenant_id: Only include historic process instances which belong to no tenant. Value may only be `true`, as `false` is the default behavior.
        :param str executed_activity_id_in: Restrict to instances that executed an activity with one of given ids. Filter by a comma-separated list of `Strings`
        :param str active_activity_id_in: Restrict to instances that have an active activity with one of given ids. Filter by a comma-separated list of `Strings`
        :param bool active: Restrict to instances that are active.
        :param bool suspended: Restrict to instances that are suspended.
        :param bool completed: Restrict to instances that are completed.
        :param bool externally_terminated: Restrict to instances that are externallyTerminated.
        :param bool internally_terminated: Restrict to instances that are internallyTerminated.
        :param str variables: Only include process instances that have/had variables with certain values. Variable filtering expressions are comma-separated and are structured as follows: A valid parameter value has the form `key_operator_value`. `key` is the variable name, `operator` is the comparison operator to be used and `value` the variable value.  **Note:** Values are always treated as String objects on server side.  Valid operator values are: `eq` - equal to; `neq` - not equal to; `gt` - greater than; `gteq` - greater than or equal to; `lt` - lower than; `lteq` - lower than or equal to; `like`.  Key and value may not contain underscore or comma characters. 
        :param bool variable_names_ignore_case: Match all variable names provided in variables case-insensitively. If set to `true` variableName and variablename are treated as equal.
        :param bool variable_values_ignore_case: Match all variable values provided in variables case-insensitively. If set to `true` variableValue and variablevalue are treated as equal.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: list[HistoricProcessInstanceDto]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.get_historic_process_instances_with_http_info(**kwargs)  # noqa: E501

    def get_historic_process_instances_with_http_info(self, **kwargs):  # noqa: E501
        """Get List  # noqa: E501

        Queries for historic process instances that fulfill the given parameters. The size of the result set can be retrieved by using the [Get Process Instance Count](https://docs.camunda.org/manual/7.14/reference/rest/history/process-instance/get-process-instance-query-count/) method.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_historic_process_instances_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str sort_by: Sort the results lexicographically by a given criterion. Must be used in conjunction with the sortOrder parameter.
        :param str sort_order: Sort the results in a given order. Values may be asc for ascending order or desc for descending order. Must be used in conjunction with the sortBy parameter.
        :param int first_result: Pagination of results. Specifies the index of the first result to return.
        :param int max_results: Pagination of results. Specifies the maximum number of results to return. Will return less results if there are no more results left.
        :param str process_instance_id: Filter by process instance id.
        :param str process_instance_ids: Filter by process instance ids. Filter by a comma-separated list of `Strings`.
        :param str process_definition_id: Filter by the process definition the instances run on.
        :param str process_definition_key: Filter by the key of the process definition the instances run on.
        :param str process_definition_key_in: Filter by a list of process definition keys. A process instance must have one of the given process definition keys. Filter by a comma-separated list of `Strings`.
        :param str process_definition_name: Filter by the name of the process definition the instances run on.
        :param str process_definition_name_like: Filter by process definition names that the parameter is a substring of.
        :param str process_definition_key_not_in: Exclude instances that belong to a set of process definitions. Filter by a comma-separated list of `Strings`.
        :param str process_instance_business_key: Filter by process instance business key.
        :param str process_instance_business_key_like: Filter by process instance business key that the parameter is a substring of.
        :param bool root_process_instances: Restrict the query to all process instances that are top level process instances.
        :param bool finished: Only include finished process instances. Value may only be `true`, as `false` is the default behavior.
        :param bool unfinished: Only include unfinished process instances. Value may only be `true`, as `false` is the default behavior.
        :param bool with_incidents: Only include process instances which have an incident. Value may only be `true`, as `false` is the default behavior.
        :param bool with_root_incidents: Only include process instances which have a root incident. Value may only be `true`, as `false` is the default behavior.
        :param str incident_type: Filter by the incident type. See the [User Guide](https://docs.camunda.org/manual/7.14/user-guide/process-engine/incidents/#incident-types) for a list of incident types.
        :param str incident_status: Only include process instances which have an incident in status either open or resolved. To get all process instances, use the query parameter withIncidents.
        :param str incident_message: Filter by the incident message. Exact match.
        :param str incident_message_like: Filter by the incident message that the parameter is a substring of.
        :param datetime started_before: Restrict to instances that were started before the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime started_after: Restrict to instances that were started after the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime finished_before: Restrict to instances that were finished before the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime finished_after: Restrict to instances that were finished after the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_activity_after: Restrict to instances that executed an activity after the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_activity_before: Restrict to instances that executed an activity before the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_job_after: Restrict to instances that executed an job after the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_job_before: Restrict to instances that executed an job before the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param str started_by: Only include process instances that were started by the given user.
        :param str super_process_instance_id: Restrict query to all process instances that are sub process instances of the given process instance. Takes a process instance id.
        :param str sub_process_instance_id: Restrict query to one process instance that has a sub process instance with the given id.
        :param str super_case_instance_id: Restrict query to all process instances that are sub process instances of the given case instance. Takes a case instance id.
        :param str sub_case_instance_id: Restrict query to one process instance that has a sub case instance with the given id.
        :param str case_instance_id: Restrict query to all process instances that are sub process instances of the given case instance. Takes a case instance id.
        :param str tenant_id_in: Filter by a list of tenant ids. A process instance must have one of the given tenant ids. Filter by a comma-separated list of `Strings`
        :param bool without_tenant_id: Only include historic process instances which belong to no tenant. Value may only be `true`, as `false` is the default behavior.
        :param str executed_activity_id_in: Restrict to instances that executed an activity with one of given ids. Filter by a comma-separated list of `Strings`
        :param str active_activity_id_in: Restrict to instances that have an active activity with one of given ids. Filter by a comma-separated list of `Strings`
        :param bool active: Restrict to instances that are active.
        :param bool suspended: Restrict to instances that are suspended.
        :param bool completed: Restrict to instances that are completed.
        :param bool externally_terminated: Restrict to instances that are externallyTerminated.
        :param bool internally_terminated: Restrict to instances that are internallyTerminated.
        :param str variables: Only include process instances that have/had variables with certain values. Variable filtering expressions are comma-separated and are structured as follows: A valid parameter value has the form `key_operator_value`. `key` is the variable name, `operator` is the comparison operator to be used and `value` the variable value.  **Note:** Values are always treated as String objects on server side.  Valid operator values are: `eq` - equal to; `neq` - not equal to; `gt` - greater than; `gteq` - greater than or equal to; `lt` - lower than; `lteq` - lower than or equal to; `like`.  Key and value may not contain underscore or comma characters. 
        :param bool variable_names_ignore_case: Match all variable names provided in variables case-insensitively. If set to `true` variableName and variablename are treated as equal.
        :param bool variable_values_ignore_case: Match all variable values provided in variables case-insensitively. If set to `true` variableValue and variablevalue are treated as equal.
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(list[HistoricProcessInstanceDto], status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'sort_by',
            'sort_order',
            'first_result',
            'max_results',
            'process_instance_id',
            'process_instance_ids',
            'process_definition_id',
            'process_definition_key',
            'process_definition_key_in',
            'process_definition_name',
            'process_definition_name_like',
            'process_definition_key_not_in',
            'process_instance_business_key',
            'process_instance_business_key_like',
            'root_process_instances',
            'finished',
            'unfinished',
            'with_incidents',
            'with_root_incidents',
            'incident_type',
            'incident_status',
            'incident_message',
            'incident_message_like',
            'started_before',
            'started_after',
            'finished_before',
            'finished_after',
            'executed_activity_after',
            'executed_activity_before',
            'executed_job_after',
            'executed_job_before',
            'started_by',
            'super_process_instance_id',
            'sub_process_instance_id',
            'super_case_instance_id',
            'sub_case_instance_id',
            'case_instance_id',
            'tenant_id_in',
            'without_tenant_id',
            'executed_activity_id_in',
            'active_activity_id_in',
            'active',
            'suspended',
            'completed',
            'externally_terminated',
            'internally_terminated',
            'variables',
            'variable_names_ignore_case',
            'variable_values_ignore_case'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_historic_process_instances" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'sort_by' in local_var_params and local_var_params['sort_by'] is not None:  # noqa: E501
            query_params.append(('sortBy', local_var_params['sort_by']))  # noqa: E501
        if 'sort_order' in local_var_params and local_var_params['sort_order'] is not None:  # noqa: E501
            query_params.append(('sortOrder', local_var_params['sort_order']))  # noqa: E501
        if 'first_result' in local_var_params and local_var_params['first_result'] is not None:  # noqa: E501
            query_params.append(('firstResult', local_var_params['first_result']))  # noqa: E501
        if 'max_results' in local_var_params and local_var_params['max_results'] is not None:  # noqa: E501
            query_params.append(('maxResults', local_var_params['max_results']))  # noqa: E501
        if 'process_instance_id' in local_var_params and local_var_params['process_instance_id'] is not None:  # noqa: E501
            query_params.append(('processInstanceId', local_var_params['process_instance_id']))  # noqa: E501
        if 'process_instance_ids' in local_var_params and local_var_params['process_instance_ids'] is not None:  # noqa: E501
            query_params.append(('processInstanceIds', local_var_params['process_instance_ids']))  # noqa: E501
        if 'process_definition_id' in local_var_params and local_var_params['process_definition_id'] is not None:  # noqa: E501
            query_params.append(('processDefinitionId', local_var_params['process_definition_id']))  # noqa: E501
        if 'process_definition_key' in local_var_params and local_var_params['process_definition_key'] is not None:  # noqa: E501
            query_params.append(('processDefinitionKey', local_var_params['process_definition_key']))  # noqa: E501
        if 'process_definition_key_in' in local_var_params and local_var_params['process_definition_key_in'] is not None:  # noqa: E501
            query_params.append(('processDefinitionKeyIn', local_var_params['process_definition_key_in']))  # noqa: E501
        if 'process_definition_name' in local_var_params and local_var_params['process_definition_name'] is not None:  # noqa: E501
            query_params.append(('processDefinitionName', local_var_params['process_definition_name']))  # noqa: E501
        if 'process_definition_name_like' in local_var_params and local_var_params['process_definition_name_like'] is not None:  # noqa: E501
            query_params.append(('processDefinitionNameLike', local_var_params['process_definition_name_like']))  # noqa: E501
        if 'process_definition_key_not_in' in local_var_params and local_var_params['process_definition_key_not_in'] is not None:  # noqa: E501
            query_params.append(('processDefinitionKeyNotIn', local_var_params['process_definition_key_not_in']))  # noqa: E501
        if 'process_instance_business_key' in local_var_params and local_var_params['process_instance_business_key'] is not None:  # noqa: E501
            query_params.append(('processInstanceBusinessKey', local_var_params['process_instance_business_key']))  # noqa: E501
        if 'process_instance_business_key_like' in local_var_params and local_var_params['process_instance_business_key_like'] is not None:  # noqa: E501
            query_params.append(('processInstanceBusinessKeyLike', local_var_params['process_instance_business_key_like']))  # noqa: E501
        if 'root_process_instances' in local_var_params and local_var_params['root_process_instances'] is not None:  # noqa: E501
            query_params.append(('rootProcessInstances', local_var_params['root_process_instances']))  # noqa: E501
        if 'finished' in local_var_params and local_var_params['finished'] is not None:  # noqa: E501
            query_params.append(('finished', local_var_params['finished']))  # noqa: E501
        if 'unfinished' in local_var_params and local_var_params['unfinished'] is not None:  # noqa: E501
            query_params.append(('unfinished', local_var_params['unfinished']))  # noqa: E501
        if 'with_incidents' in local_var_params and local_var_params['with_incidents'] is not None:  # noqa: E501
            query_params.append(('withIncidents', local_var_params['with_incidents']))  # noqa: E501
        if 'with_root_incidents' in local_var_params and local_var_params['with_root_incidents'] is not None:  # noqa: E501
            query_params.append(('withRootIncidents', local_var_params['with_root_incidents']))  # noqa: E501
        if 'incident_type' in local_var_params and local_var_params['incident_type'] is not None:  # noqa: E501
            query_params.append(('incidentType', local_var_params['incident_type']))  # noqa: E501
        if 'incident_status' in local_var_params and local_var_params['incident_status'] is not None:  # noqa: E501
            query_params.append(('incidentStatus', local_var_params['incident_status']))  # noqa: E501
        if 'incident_message' in local_var_params and local_var_params['incident_message'] is not None:  # noqa: E501
            query_params.append(('incidentMessage', local_var_params['incident_message']))  # noqa: E501
        if 'incident_message_like' in local_var_params and local_var_params['incident_message_like'] is not None:  # noqa: E501
            query_params.append(('incidentMessageLike', local_var_params['incident_message_like']))  # noqa: E501
        if 'started_before' in local_var_params and local_var_params['started_before'] is not None:  # noqa: E501
            query_params.append(('startedBefore', local_var_params['started_before']))  # noqa: E501
        if 'started_after' in local_var_params and local_var_params['started_after'] is not None:  # noqa: E501
            query_params.append(('startedAfter', local_var_params['started_after']))  # noqa: E501
        if 'finished_before' in local_var_params and local_var_params['finished_before'] is not None:  # noqa: E501
            query_params.append(('finishedBefore', local_var_params['finished_before']))  # noqa: E501
        if 'finished_after' in local_var_params and local_var_params['finished_after'] is not None:  # noqa: E501
            query_params.append(('finishedAfter', local_var_params['finished_after']))  # noqa: E501
        if 'executed_activity_after' in local_var_params and local_var_params['executed_activity_after'] is not None:  # noqa: E501
            query_params.append(('executedActivityAfter', local_var_params['executed_activity_after']))  # noqa: E501
        if 'executed_activity_before' in local_var_params and local_var_params['executed_activity_before'] is not None:  # noqa: E501
            query_params.append(('executedActivityBefore', local_var_params['executed_activity_before']))  # noqa: E501
        if 'executed_job_after' in local_var_params and local_var_params['executed_job_after'] is not None:  # noqa: E501
            query_params.append(('executedJobAfter', local_var_params['executed_job_after']))  # noqa: E501
        if 'executed_job_before' in local_var_params and local_var_params['executed_job_before'] is not None:  # noqa: E501
            query_params.append(('executedJobBefore', local_var_params['executed_job_before']))  # noqa: E501
        if 'started_by' in local_var_params and local_var_params['started_by'] is not None:  # noqa: E501
            query_params.append(('startedBy', local_var_params['started_by']))  # noqa: E501
        if 'super_process_instance_id' in local_var_params and local_var_params['super_process_instance_id'] is not None:  # noqa: E501
            query_params.append(('superProcessInstanceId', local_var_params['super_process_instance_id']))  # noqa: E501
        if 'sub_process_instance_id' in local_var_params and local_var_params['sub_process_instance_id'] is not None:  # noqa: E501
            query_params.append(('subProcessInstanceId', local_var_params['sub_process_instance_id']))  # noqa: E501
        if 'super_case_instance_id' in local_var_params and local_var_params['super_case_instance_id'] is not None:  # noqa: E501
            query_params.append(('superCaseInstanceId', local_var_params['super_case_instance_id']))  # noqa: E501
        if 'sub_case_instance_id' in local_var_params and local_var_params['sub_case_instance_id'] is not None:  # noqa: E501
            query_params.append(('subCaseInstanceId', local_var_params['sub_case_instance_id']))  # noqa: E501
        if 'case_instance_id' in local_var_params and local_var_params['case_instance_id'] is not None:  # noqa: E501
            query_params.append(('caseInstanceId', local_var_params['case_instance_id']))  # noqa: E501
        if 'tenant_id_in' in local_var_params and local_var_params['tenant_id_in'] is not None:  # noqa: E501
            query_params.append(('tenantIdIn', local_var_params['tenant_id_in']))  # noqa: E501
        if 'without_tenant_id' in local_var_params and local_var_params['without_tenant_id'] is not None:  # noqa: E501
            query_params.append(('withoutTenantId', local_var_params['without_tenant_id']))  # noqa: E501
        if 'executed_activity_id_in' in local_var_params and local_var_params['executed_activity_id_in'] is not None:  # noqa: E501
            query_params.append(('executedActivityIdIn', local_var_params['executed_activity_id_in']))  # noqa: E501
        if 'active_activity_id_in' in local_var_params and local_var_params['active_activity_id_in'] is not None:  # noqa: E501
            query_params.append(('activeActivityIdIn', local_var_params['active_activity_id_in']))  # noqa: E501
        if 'active' in local_var_params and local_var_params['active'] is not None:  # noqa: E501
            query_params.append(('active', local_var_params['active']))  # noqa: E501
        if 'suspended' in local_var_params and local_var_params['suspended'] is not None:  # noqa: E501
            query_params.append(('suspended', local_var_params['suspended']))  # noqa: E501
        if 'completed' in local_var_params and local_var_params['completed'] is not None:  # noqa: E501
            query_params.append(('completed', local_var_params['completed']))  # noqa: E501
        if 'externally_terminated' in local_var_params and local_var_params['externally_terminated'] is not None:  # noqa: E501
            query_params.append(('externallyTerminated', local_var_params['externally_terminated']))  # noqa: E501
        if 'internally_terminated' in local_var_params and local_var_params['internally_terminated'] is not None:  # noqa: E501
            query_params.append(('internallyTerminated', local_var_params['internally_terminated']))  # noqa: E501
        if 'variables' in local_var_params and local_var_params['variables'] is not None:  # noqa: E501
            query_params.append(('variables', local_var_params['variables']))  # noqa: E501
        if 'variable_names_ignore_case' in local_var_params and local_var_params['variable_names_ignore_case'] is not None:  # noqa: E501
            query_params.append(('variableNamesIgnoreCase', local_var_params['variable_names_ignore_case']))  # noqa: E501
        if 'variable_values_ignore_case' in local_var_params and local_var_params['variable_values_ignore_case'] is not None:  # noqa: E501
            query_params.append(('variableValuesIgnoreCase', local_var_params['variable_values_ignore_case']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/history/process-instance', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[HistoricProcessInstanceDto]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_historic_process_instances_count(self, **kwargs):  # noqa: E501
        """Get List Count  # noqa: E501

        Queries for the number of historic process instances that fulfill the given parameters. Takes the same parameters as the [Get Process Instances](https://docs.camunda.org/manual/7.14/reference/rest/history/process-instance/get-process-instance-query/) method.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_historic_process_instances_count(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str process_instance_id: Filter by process instance id.
        :param str process_instance_ids: Filter by process instance ids. Filter by a comma-separated list of `Strings`.
        :param str process_definition_id: Filter by the process definition the instances run on.
        :param str process_definition_key: Filter by the key of the process definition the instances run on.
        :param str process_definition_key_in: Filter by a list of process definition keys. A process instance must have one of the given process definition keys. Filter by a comma-separated list of `Strings`.
        :param str process_definition_name: Filter by the name of the process definition the instances run on.
        :param str process_definition_name_like: Filter by process definition names that the parameter is a substring of.
        :param str process_definition_key_not_in: Exclude instances that belong to a set of process definitions. Filter by a comma-separated list of `Strings`.
        :param str process_instance_business_key: Filter by process instance business key.
        :param str process_instance_business_key_like: Filter by process instance business key that the parameter is a substring of.
        :param bool root_process_instances: Restrict the query to all process instances that are top level process instances.
        :param bool finished: Only include finished process instances. Value may only be `true`, as `false` is the default behavior.
        :param bool unfinished: Only include unfinished process instances. Value may only be `true`, as `false` is the default behavior.
        :param bool with_incidents: Only include process instances which have an incident. Value may only be `true`, as `false` is the default behavior.
        :param bool with_root_incidents: Only include process instances which have a root incident. Value may only be `true`, as `false` is the default behavior.
        :param str incident_type: Filter by the incident type. See the [User Guide](https://docs.camunda.org/manual/7.14/user-guide/process-engine/incidents/#incident-types) for a list of incident types.
        :param str incident_status: Only include process instances which have an incident in status either open or resolved. To get all process instances, use the query parameter withIncidents.
        :param str incident_message: Filter by the incident message. Exact match.
        :param str incident_message_like: Filter by the incident message that the parameter is a substring of.
        :param datetime started_before: Restrict to instances that were started before the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime started_after: Restrict to instances that were started after the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime finished_before: Restrict to instances that were finished before the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime finished_after: Restrict to instances that were finished after the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_activity_after: Restrict to instances that executed an activity after the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_activity_before: Restrict to instances that executed an activity before the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_job_after: Restrict to instances that executed an job after the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_job_before: Restrict to instances that executed an job before the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param str started_by: Only include process instances that were started by the given user.
        :param str super_process_instance_id: Restrict query to all process instances that are sub process instances of the given process instance. Takes a process instance id.
        :param str sub_process_instance_id: Restrict query to one process instance that has a sub process instance with the given id.
        :param str super_case_instance_id: Restrict query to all process instances that are sub process instances of the given case instance. Takes a case instance id.
        :param str sub_case_instance_id: Restrict query to one process instance that has a sub case instance with the given id.
        :param str case_instance_id: Restrict query to all process instances that are sub process instances of the given case instance. Takes a case instance id.
        :param str tenant_id_in: Filter by a list of tenant ids. A process instance must have one of the given tenant ids. Filter by a comma-separated list of `Strings`
        :param bool without_tenant_id: Only include historic process instances which belong to no tenant. Value may only be `true`, as `false` is the default behavior.
        :param str executed_activity_id_in: Restrict to instances that executed an activity with one of given ids. Filter by a comma-separated list of `Strings`
        :param str active_activity_id_in: Restrict to instances that have an active activity with one of given ids. Filter by a comma-separated list of `Strings`
        :param bool active: Restrict to instances that are active.
        :param bool suspended: Restrict to instances that are suspended.
        :param bool completed: Restrict to instances that are completed.
        :param bool externally_terminated: Restrict to instances that are externallyTerminated.
        :param bool internally_terminated: Restrict to instances that are internallyTerminated.
        :param str variables: Only include process instances that have/had variables with certain values. Variable filtering expressions are comma-separated and are structured as follows: A valid parameter value has the form `key_operator_value`. `key` is the variable name, `operator` is the comparison operator to be used and `value` the variable value.  **Note:** Values are always treated as String objects on server side.  Valid operator values are: `eq` - equal to; `neq` - not equal to; `gt` - greater than; `gteq` - greater than or equal to; `lt` - lower than; `lteq` - lower than or equal to; `like`.  Key and value may not contain underscore or comma characters. 
        :param bool variable_names_ignore_case: Match all variable names provided in variables case-insensitively. If set to `true` variableName and variablename are treated as equal.
        :param bool variable_values_ignore_case: Match all variable values provided in variables case-insensitively. If set to `true` variableValue and variablevalue are treated as equal.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: CountResultDto
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.get_historic_process_instances_count_with_http_info(**kwargs)  # noqa: E501

    def get_historic_process_instances_count_with_http_info(self, **kwargs):  # noqa: E501
        """Get List Count  # noqa: E501

        Queries for the number of historic process instances that fulfill the given parameters. Takes the same parameters as the [Get Process Instances](https://docs.camunda.org/manual/7.14/reference/rest/history/process-instance/get-process-instance-query/) method.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_historic_process_instances_count_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str process_instance_id: Filter by process instance id.
        :param str process_instance_ids: Filter by process instance ids. Filter by a comma-separated list of `Strings`.
        :param str process_definition_id: Filter by the process definition the instances run on.
        :param str process_definition_key: Filter by the key of the process definition the instances run on.
        :param str process_definition_key_in: Filter by a list of process definition keys. A process instance must have one of the given process definition keys. Filter by a comma-separated list of `Strings`.
        :param str process_definition_name: Filter by the name of the process definition the instances run on.
        :param str process_definition_name_like: Filter by process definition names that the parameter is a substring of.
        :param str process_definition_key_not_in: Exclude instances that belong to a set of process definitions. Filter by a comma-separated list of `Strings`.
        :param str process_instance_business_key: Filter by process instance business key.
        :param str process_instance_business_key_like: Filter by process instance business key that the parameter is a substring of.
        :param bool root_process_instances: Restrict the query to all process instances that are top level process instances.
        :param bool finished: Only include finished process instances. Value may only be `true`, as `false` is the default behavior.
        :param bool unfinished: Only include unfinished process instances. Value may only be `true`, as `false` is the default behavior.
        :param bool with_incidents: Only include process instances which have an incident. Value may only be `true`, as `false` is the default behavior.
        :param bool with_root_incidents: Only include process instances which have a root incident. Value may only be `true`, as `false` is the default behavior.
        :param str incident_type: Filter by the incident type. See the [User Guide](https://docs.camunda.org/manual/7.14/user-guide/process-engine/incidents/#incident-types) for a list of incident types.
        :param str incident_status: Only include process instances which have an incident in status either open or resolved. To get all process instances, use the query parameter withIncidents.
        :param str incident_message: Filter by the incident message. Exact match.
        :param str incident_message_like: Filter by the incident message that the parameter is a substring of.
        :param datetime started_before: Restrict to instances that were started before the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime started_after: Restrict to instances that were started after the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime finished_before: Restrict to instances that were finished before the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime finished_after: Restrict to instances that were finished after the given date. By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_activity_after: Restrict to instances that executed an activity after the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_activity_before: Restrict to instances that executed an activity before the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_job_after: Restrict to instances that executed an job after the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param datetime executed_job_before: Restrict to instances that executed an job before the given date (inclusive). By [default](https://docs.camunda.org/manual/7.14/reference/rest/overview/date-format/), the date must have the format `yyyy-MM-dd'T'HH:mm:ss.SSSZ`, e.g., `2013-01-23T14:42:45.000+0200`.
        :param str started_by: Only include process instances that were started by the given user.
        :param str super_process_instance_id: Restrict query to all process instances that are sub process instances of the given process instance. Takes a process instance id.
        :param str sub_process_instance_id: Restrict query to one process instance that has a sub process instance with the given id.
        :param str super_case_instance_id: Restrict query to all process instances that are sub process instances of the given case instance. Takes a case instance id.
        :param str sub_case_instance_id: Restrict query to one process instance that has a sub case instance with the given id.
        :param str case_instance_id: Restrict query to all process instances that are sub process instances of the given case instance. Takes a case instance id.
        :param str tenant_id_in: Filter by a list of tenant ids. A process instance must have one of the given tenant ids. Filter by a comma-separated list of `Strings`
        :param bool without_tenant_id: Only include historic process instances which belong to no tenant. Value may only be `true`, as `false` is the default behavior.
        :param str executed_activity_id_in: Restrict to instances that executed an activity with one of given ids. Filter by a comma-separated list of `Strings`
        :param str active_activity_id_in: Restrict to instances that have an active activity with one of given ids. Filter by a comma-separated list of `Strings`
        :param bool active: Restrict to instances that are active.
        :param bool suspended: Restrict to instances that are suspended.
        :param bool completed: Restrict to instances that are completed.
        :param bool externally_terminated: Restrict to instances that are externallyTerminated.
        :param bool internally_terminated: Restrict to instances that are internallyTerminated.
        :param str variables: Only include process instances that have/had variables with certain values. Variable filtering expressions are comma-separated and are structured as follows: A valid parameter value has the form `key_operator_value`. `key` is the variable name, `operator` is the comparison operator to be used and `value` the variable value.  **Note:** Values are always treated as String objects on server side.  Valid operator values are: `eq` - equal to; `neq` - not equal to; `gt` - greater than; `gteq` - greater than or equal to; `lt` - lower than; `lteq` - lower than or equal to; `like`.  Key and value may not contain underscore or comma characters. 
        :param bool variable_names_ignore_case: Match all variable names provided in variables case-insensitively. If set to `true` variableName and variablename are treated as equal.
        :param bool variable_values_ignore_case: Match all variable values provided in variables case-insensitively. If set to `true` variableValue and variablevalue are treated as equal.
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(CountResultDto, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'process_instance_id',
            'process_instance_ids',
            'process_definition_id',
            'process_definition_key',
            'process_definition_key_in',
            'process_definition_name',
            'process_definition_name_like',
            'process_definition_key_not_in',
            'process_instance_business_key',
            'process_instance_business_key_like',
            'root_process_instances',
            'finished',
            'unfinished',
            'with_incidents',
            'with_root_incidents',
            'incident_type',
            'incident_status',
            'incident_message',
            'incident_message_like',
            'started_before',
            'started_after',
            'finished_before',
            'finished_after',
            'executed_activity_after',
            'executed_activity_before',
            'executed_job_after',
            'executed_job_before',
            'started_by',
            'super_process_instance_id',
            'sub_process_instance_id',
            'super_case_instance_id',
            'sub_case_instance_id',
            'case_instance_id',
            'tenant_id_in',
            'without_tenant_id',
            'executed_activity_id_in',
            'active_activity_id_in',
            'active',
            'suspended',
            'completed',
            'externally_terminated',
            'internally_terminated',
            'variables',
            'variable_names_ignore_case',
            'variable_values_ignore_case'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_historic_process_instances_count" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'process_instance_id' in local_var_params and local_var_params['process_instance_id'] is not None:  # noqa: E501
            query_params.append(('processInstanceId', local_var_params['process_instance_id']))  # noqa: E501
        if 'process_instance_ids' in local_var_params and local_var_params['process_instance_ids'] is not None:  # noqa: E501
            query_params.append(('processInstanceIds', local_var_params['process_instance_ids']))  # noqa: E501
        if 'process_definition_id' in local_var_params and local_var_params['process_definition_id'] is not None:  # noqa: E501
            query_params.append(('processDefinitionId', local_var_params['process_definition_id']))  # noqa: E501
        if 'process_definition_key' in local_var_params and local_var_params['process_definition_key'] is not None:  # noqa: E501
            query_params.append(('processDefinitionKey', local_var_params['process_definition_key']))  # noqa: E501
        if 'process_definition_key_in' in local_var_params and local_var_params['process_definition_key_in'] is not None:  # noqa: E501
            query_params.append(('processDefinitionKeyIn', local_var_params['process_definition_key_in']))  # noqa: E501
        if 'process_definition_name' in local_var_params and local_var_params['process_definition_name'] is not None:  # noqa: E501
            query_params.append(('processDefinitionName', local_var_params['process_definition_name']))  # noqa: E501
        if 'process_definition_name_like' in local_var_params and local_var_params['process_definition_name_like'] is not None:  # noqa: E501
            query_params.append(('processDefinitionNameLike', local_var_params['process_definition_name_like']))  # noqa: E501
        if 'process_definition_key_not_in' in local_var_params and local_var_params['process_definition_key_not_in'] is not None:  # noqa: E501
            query_params.append(('processDefinitionKeyNotIn', local_var_params['process_definition_key_not_in']))  # noqa: E501
        if 'process_instance_business_key' in local_var_params and local_var_params['process_instance_business_key'] is not None:  # noqa: E501
            query_params.append(('processInstanceBusinessKey', local_var_params['process_instance_business_key']))  # noqa: E501
        if 'process_instance_business_key_like' in local_var_params and local_var_params['process_instance_business_key_like'] is not None:  # noqa: E501
            query_params.append(('processInstanceBusinessKeyLike', local_var_params['process_instance_business_key_like']))  # noqa: E501
        if 'root_process_instances' in local_var_params and local_var_params['root_process_instances'] is not None:  # noqa: E501
            query_params.append(('rootProcessInstances', local_var_params['root_process_instances']))  # noqa: E501
        if 'finished' in local_var_params and local_var_params['finished'] is not None:  # noqa: E501
            query_params.append(('finished', local_var_params['finished']))  # noqa: E501
        if 'unfinished' in local_var_params and local_var_params['unfinished'] is not None:  # noqa: E501
            query_params.append(('unfinished', local_var_params['unfinished']))  # noqa: E501
        if 'with_incidents' in local_var_params and local_var_params['with_incidents'] is not None:  # noqa: E501
            query_params.append(('withIncidents', local_var_params['with_incidents']))  # noqa: E501
        if 'with_root_incidents' in local_var_params and local_var_params['with_root_incidents'] is not None:  # noqa: E501
            query_params.append(('withRootIncidents', local_var_params['with_root_incidents']))  # noqa: E501
        if 'incident_type' in local_var_params and local_var_params['incident_type'] is not None:  # noqa: E501
            query_params.append(('incidentType', local_var_params['incident_type']))  # noqa: E501
        if 'incident_status' in local_var_params and local_var_params['incident_status'] is not None:  # noqa: E501
            query_params.append(('incidentStatus', local_var_params['incident_status']))  # noqa: E501
        if 'incident_message' in local_var_params and local_var_params['incident_message'] is not None:  # noqa: E501
            query_params.append(('incidentMessage', local_var_params['incident_message']))  # noqa: E501
        if 'incident_message_like' in local_var_params and local_var_params['incident_message_like'] is not None:  # noqa: E501
            query_params.append(('incidentMessageLike', local_var_params['incident_message_like']))  # noqa: E501
        if 'started_before' in local_var_params and local_var_params['started_before'] is not None:  # noqa: E501
            query_params.append(('startedBefore', local_var_params['started_before']))  # noqa: E501
        if 'started_after' in local_var_params and local_var_params['started_after'] is not None:  # noqa: E501
            query_params.append(('startedAfter', local_var_params['started_after']))  # noqa: E501
        if 'finished_before' in local_var_params and local_var_params['finished_before'] is not None:  # noqa: E501
            query_params.append(('finishedBefore', local_var_params['finished_before']))  # noqa: E501
        if 'finished_after' in local_var_params and local_var_params['finished_after'] is not None:  # noqa: E501
            query_params.append(('finishedAfter', local_var_params['finished_after']))  # noqa: E501
        if 'executed_activity_after' in local_var_params and local_var_params['executed_activity_after'] is not None:  # noqa: E501
            query_params.append(('executedActivityAfter', local_var_params['executed_activity_after']))  # noqa: E501
        if 'executed_activity_before' in local_var_params and local_var_params['executed_activity_before'] is not None:  # noqa: E501
            query_params.append(('executedActivityBefore', local_var_params['executed_activity_before']))  # noqa: E501
        if 'executed_job_after' in local_var_params and local_var_params['executed_job_after'] is not None:  # noqa: E501
            query_params.append(('executedJobAfter', local_var_params['executed_job_after']))  # noqa: E501
        if 'executed_job_before' in local_var_params and local_var_params['executed_job_before'] is not None:  # noqa: E501
            query_params.append(('executedJobBefore', local_var_params['executed_job_before']))  # noqa: E501
        if 'started_by' in local_var_params and local_var_params['started_by'] is not None:  # noqa: E501
            query_params.append(('startedBy', local_var_params['started_by']))  # noqa: E501
        if 'super_process_instance_id' in local_var_params and local_var_params['super_process_instance_id'] is not None:  # noqa: E501
            query_params.append(('superProcessInstanceId', local_var_params['super_process_instance_id']))  # noqa: E501
        if 'sub_process_instance_id' in local_var_params and local_var_params['sub_process_instance_id'] is not None:  # noqa: E501
            query_params.append(('subProcessInstanceId', local_var_params['sub_process_instance_id']))  # noqa: E501
        if 'super_case_instance_id' in local_var_params and local_var_params['super_case_instance_id'] is not None:  # noqa: E501
            query_params.append(('superCaseInstanceId', local_var_params['super_case_instance_id']))  # noqa: E501
        if 'sub_case_instance_id' in local_var_params and local_var_params['sub_case_instance_id'] is not None:  # noqa: E501
            query_params.append(('subCaseInstanceId', local_var_params['sub_case_instance_id']))  # noqa: E501
        if 'case_instance_id' in local_var_params and local_var_params['case_instance_id'] is not None:  # noqa: E501
            query_params.append(('caseInstanceId', local_var_params['case_instance_id']))  # noqa: E501
        if 'tenant_id_in' in local_var_params and local_var_params['tenant_id_in'] is not None:  # noqa: E501
            query_params.append(('tenantIdIn', local_var_params['tenant_id_in']))  # noqa: E501
        if 'without_tenant_id' in local_var_params and local_var_params['without_tenant_id'] is not None:  # noqa: E501
            query_params.append(('withoutTenantId', local_var_params['without_tenant_id']))  # noqa: E501
        if 'executed_activity_id_in' in local_var_params and local_var_params['executed_activity_id_in'] is not None:  # noqa: E501
            query_params.append(('executedActivityIdIn', local_var_params['executed_activity_id_in']))  # noqa: E501
        if 'active_activity_id_in' in local_var_params and local_var_params['active_activity_id_in'] is not None:  # noqa: E501
            query_params.append(('activeActivityIdIn', local_var_params['active_activity_id_in']))  # noqa: E501
        if 'active' in local_var_params and local_var_params['active'] is not None:  # noqa: E501
            query_params.append(('active', local_var_params['active']))  # noqa: E501
        if 'suspended' in local_var_params and local_var_params['suspended'] is not None:  # noqa: E501
            query_params.append(('suspended', local_var_params['suspended']))  # noqa: E501
        if 'completed' in local_var_params and local_var_params['completed'] is not None:  # noqa: E501
            query_params.append(('completed', local_var_params['completed']))  # noqa: E501
        if 'externally_terminated' in local_var_params and local_var_params['externally_terminated'] is not None:  # noqa: E501
            query_params.append(('externallyTerminated', local_var_params['externally_terminated']))  # noqa: E501
        if 'internally_terminated' in local_var_params and local_var_params['internally_terminated'] is not None:  # noqa: E501
            query_params.append(('internallyTerminated', local_var_params['internally_terminated']))  # noqa: E501
        if 'variables' in local_var_params and local_var_params['variables'] is not None:  # noqa: E501
            query_params.append(('variables', local_var_params['variables']))  # noqa: E501
        if 'variable_names_ignore_case' in local_var_params and local_var_params['variable_names_ignore_case'] is not None:  # noqa: E501
            query_params.append(('variableNamesIgnoreCase', local_var_params['variable_names_ignore_case']))  # noqa: E501
        if 'variable_values_ignore_case' in local_var_params and local_var_params['variable_values_ignore_case'] is not None:  # noqa: E501
            query_params.append(('variableValuesIgnoreCase', local_var_params['variable_values_ignore_case']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/history/process-instance/count', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CountResultDto',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def query_historic_process_instances(self, **kwargs):  # noqa: E501
        """Get List (POST)  # noqa: E501

        Queries for historic process instances that fulfill the given parameters. This method is slightly more powerful than the [Get Process Instance](https://docs.camunda.org/manual/7.14/reference/rest/history/process-instance/get-process-instance-query/) because it allows filtering by multiple process variables of types `String`, `Number` or `Boolean`.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.query_historic_process_instances(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int first_result: Pagination of results. Specifies the index of the first result to return.
        :param int max_results: Pagination of results. Specifies the maximum number of results to return. Will return less results if there are no more results left.
        :param HistoricProcessInstanceQueryDto historic_process_instance_query_dto:
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: list[HistoricProcessInstanceDto]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.query_historic_process_instances_with_http_info(**kwargs)  # noqa: E501

    def query_historic_process_instances_with_http_info(self, **kwargs):  # noqa: E501
        """Get List (POST)  # noqa: E501

        Queries for historic process instances that fulfill the given parameters. This method is slightly more powerful than the [Get Process Instance](https://docs.camunda.org/manual/7.14/reference/rest/history/process-instance/get-process-instance-query/) because it allows filtering by multiple process variables of types `String`, `Number` or `Boolean`.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.query_historic_process_instances_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int first_result: Pagination of results. Specifies the index of the first result to return.
        :param int max_results: Pagination of results. Specifies the maximum number of results to return. Will return less results if there are no more results left.
        :param HistoricProcessInstanceQueryDto historic_process_instance_query_dto:
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(list[HistoricProcessInstanceDto], status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'first_result',
            'max_results',
            'historic_process_instance_query_dto'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method query_historic_process_instances" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'first_result' in local_var_params and local_var_params['first_result'] is not None:  # noqa: E501
            query_params.append(('firstResult', local_var_params['first_result']))  # noqa: E501
        if 'max_results' in local_var_params and local_var_params['max_results'] is not None:  # noqa: E501
            query_params.append(('maxResults', local_var_params['max_results']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'historic_process_instance_query_dto' in local_var_params:
            body_params = local_var_params['historic_process_instance_query_dto']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/history/process-instance', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[HistoricProcessInstanceDto]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def query_historic_process_instances_count(self, **kwargs):  # noqa: E501
        """Get List Count (POST)  # noqa: E501

        Queries for the number of historic process instances that fulfill the given parameters. This method takes the same message body as the [Get Process Instances (POST)](https://docs.camunda.org/manual/7.14/reference/rest/history/process-instance/get-process-instance-query/) method and therefore it is slightly more powerful than the [Get Process Instance Count](https://docs.camunda.org/manual/7.14/reference/rest/history/process-instance/post-process-instance-query-count/) method.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.query_historic_process_instances_count(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param HistoricProcessInstanceQueryDto historic_process_instance_query_dto:
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: CountResultDto
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.query_historic_process_instances_count_with_http_info(**kwargs)  # noqa: E501

    def query_historic_process_instances_count_with_http_info(self, **kwargs):  # noqa: E501
        """Get List Count (POST)  # noqa: E501

        Queries for the number of historic process instances that fulfill the given parameters. This method takes the same message body as the [Get Process Instances (POST)](https://docs.camunda.org/manual/7.14/reference/rest/history/process-instance/get-process-instance-query/) method and therefore it is slightly more powerful than the [Get Process Instance Count](https://docs.camunda.org/manual/7.14/reference/rest/history/process-instance/post-process-instance-query-count/) method.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.query_historic_process_instances_count_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param HistoricProcessInstanceQueryDto historic_process_instance_query_dto:
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(CountResultDto, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'historic_process_instance_query_dto'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method query_historic_process_instances_count" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'historic_process_instance_query_dto' in local_var_params:
            body_params = local_var_params['historic_process_instance_query_dto']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/history/process-instance/count', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CountResultDto',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def set_removal_time_async(self, **kwargs):  # noqa: E501
        """Set Removal Time Async (POST)  # noqa: E501

        Sets the removal time to multiple historic process instances asynchronously (batch).  At least `historicProcessInstanceIds` or `historicProcessInstanceQuery` has to be provided. If both are provided, all instances matching query criterion and instances from the list will be updated with a removal time.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.set_removal_time_async(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param SetRemovalTimeToHistoricProcessInstancesDto set_removal_time_to_historic_process_instances_dto:
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: BatchDto
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.set_removal_time_async_with_http_info(**kwargs)  # noqa: E501

    def set_removal_time_async_with_http_info(self, **kwargs):  # noqa: E501
        """Set Removal Time Async (POST)  # noqa: E501

        Sets the removal time to multiple historic process instances asynchronously (batch).  At least `historicProcessInstanceIds` or `historicProcessInstanceQuery` has to be provided. If both are provided, all instances matching query criterion and instances from the list will be updated with a removal time.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.set_removal_time_async_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param SetRemovalTimeToHistoricProcessInstancesDto set_removal_time_to_historic_process_instances_dto:
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(BatchDto, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'set_removal_time_to_historic_process_instances_dto'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_removal_time_async" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'set_removal_time_to_historic_process_instances_dto' in local_var_params:
            body_params = local_var_params['set_removal_time_to_historic_process_instances_dto']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/history/process-instance/set-removal-time', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='BatchDto',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)
